@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,15 +13,15 @@
 
 #include <slepc/private/dsimpl.h>      /*I "slepcds.h" I*/
 
-PetscFunctionList DSList = 0;
+PetscFunctionList DSList = NULL;
 PetscBool         DSRegisterAllCalled = PETSC_FALSE;
 PetscClassId      DS_CLASSID = 0;
 PetscLogEvent     DS_Solve = 0,DS_Vectors = 0,DS_Synchronize = 0,DS_Other = 0;
 static PetscBool  DSPackageInitialized = PETSC_FALSE;
 
-const char *DSStateTypes[] = {"RAW","INTERMEDIATE","CONDENSED","TRUNCATED","DSStateType","DS_STATE_",0};
-const char *DSParallelTypes[] = {"REDUNDANT","SYNCHRONIZED","DSParallelType","DS_PARALLEL_",0};
-const char *DSMatName[DS_NUM_MAT] = {"A","B","C","T","D","Q","Z","X","Y","U","VT","W","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9"};
+const char *DSStateTypes[] = {"RAW","INTERMEDIATE","CONDENSED","TRUNCATED","DSStateType","DS_STATE_",NULL};
+const char *DSParallelTypes[] = {"REDUNDANT","SYNCHRONIZED","DISTRIBUTED","DSParallelType","DS_PARALLEL_",NULL};
+const char *DSMatName[DS_NUM_MAT] = {"A","B","C","T","D","Q","Z","X","Y","U","V","W","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9"};
 DSMatType  DSMatExtra[DS_NUM_EXTRA] = {DS_MAT_E0,DS_MAT_E1,DS_MAT_E2,DS_MAT_E3,DS_MAT_E4,DS_MAT_E5,DS_MAT_E6,DS_MAT_E7,DS_MAT_E8,DS_MAT_E9};
 
 /*@C
@@ -34,10 +34,8 @@ DSMatType  DSMatExtra[DS_NUM_EXTRA] = {DS_MAT_E0,DS_MAT_E1,DS_MAT_E2,DS_MAT_E3,D
 @*/
 PetscErrorCode DSFinalizePackage(void)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
-  ierr = PetscFunctionListDestroy(&DSList);CHKERRQ(ierr);
+  PetscCall(PetscFunctionListDestroy(&DSList));
   DSPackageInitialized = PETSC_FALSE;
   DSRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
@@ -57,31 +55,30 @@ PetscErrorCode DSInitializePackage()
   char           logList[256];
   PetscBool      opt,pkg;
   PetscClassId   classids[1];
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (DSPackageInitialized) PetscFunctionReturn(0);
   DSPackageInitialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscClassIdRegister("Direct Solver",&DS_CLASSID);CHKERRQ(ierr);
+  PetscCall(PetscClassIdRegister("Direct Solver",&DS_CLASSID));
   /* Register Constructors */
-  ierr = DSRegisterAll();CHKERRQ(ierr);
+  PetscCall(DSRegisterAll());
   /* Register Events */
-  ierr = PetscLogEventRegister("DSSolve",DS_CLASSID,&DS_Solve);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("DSVectors",DS_CLASSID,&DS_Vectors);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("DSSynchronize",DS_CLASSID,&DS_Synchronize);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("DSOther",DS_CLASSID,&DS_Other);CHKERRQ(ierr);
+  PetscCall(PetscLogEventRegister("DSSolve",DS_CLASSID,&DS_Solve));
+  PetscCall(PetscLogEventRegister("DSVectors",DS_CLASSID,&DS_Vectors));
+  PetscCall(PetscLogEventRegister("DSSynchronize",DS_CLASSID,&DS_Synchronize));
+  PetscCall(PetscLogEventRegister("DSOther",DS_CLASSID,&DS_Other));
   /* Process Info */
   classids[0] = DS_CLASSID;
-  ierr = PetscInfoProcessClass("ds",1,&classids[0]);CHKERRQ(ierr);
+  PetscCall(PetscInfoProcessClass("ds",1,&classids[0]));
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
+  PetscCall(PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt));
   if (opt) {
-    ierr = PetscStrInList("ds",logList,',',&pkg);CHKERRQ(ierr);
-    if (pkg) { ierr = PetscLogEventDeactivateClass(DS_CLASSID);CHKERRQ(ierr); }
+    PetscCall(PetscStrInList("ds",logList,',',&pkg));
+    if (pkg) PetscCall(PetscLogEventDeactivateClass(DS_CLASSID));
   }
   /* Register package finalizer */
-  ierr = PetscRegisterFinalize(DSFinalizePackage);CHKERRQ(ierr);
+  PetscCall(PetscRegisterFinalize(DSFinalizePackage));
   PetscFunctionReturn(0);
 }
 
@@ -108,13 +105,12 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
 {
   DS             ds;
   PetscInt       i;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidPointer(newds,2);
-  *newds = 0;
-  ierr = DSInitializePackage();CHKERRQ(ierr);
-  ierr = SlepcHeaderCreate(ds,DS_CLASSID,"DS","Direct Solver (or Dense System)","DS",comm,DSDestroy,DSView);CHKERRQ(ierr);
+  *newds = NULL;
+  PetscCall(DSInitializePackage());
+  PetscCall(SlepcHeaderCreate(ds,DS_CLASSID,"DS","Direct Solver (or Dense System)","DS",comm,DSDestroy,DSView));
 
   ds->state         = DS_STATE_RAW;
   ds->method        = 0;
@@ -124,18 +120,13 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
   ds->ld            = 0;
   ds->l             = 0;
   ds->n             = 0;
-  ds->m             = 0;
   ds->k             = 0;
   ds->t             = 0;
   ds->bs            = 1;
   ds->sc            = NULL;
   ds->pmode         = DS_PARALLEL_REDUNDANT;
 
-  for (i=0;i<DS_NUM_MAT;i++) {
-    ds->mat[i]      = NULL;
-    ds->rmat[i]     = NULL;
-    ds->omat[i]     = NULL;
-  }
+  for (i=0;i<DS_NUM_MAT;i++) ds->omat[i] = NULL;
   ds->perm          = NULL;
   ds->data          = NULL;
   ds->scset         = PETSC_FALSE;
@@ -171,11 +162,9 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
 @*/
 PetscErrorCode DSSetOptionsPrefix(DS ds,const char *prefix)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  ierr = PetscObjectSetOptionsPrefix((PetscObject)ds,prefix);CHKERRQ(ierr);
+  PetscCall(PetscObjectSetOptionsPrefix((PetscObject)ds,prefix));
   PetscFunctionReturn(0);
 }
 
@@ -199,11 +188,9 @@ PetscErrorCode DSSetOptionsPrefix(DS ds,const char *prefix)
 @*/
 PetscErrorCode DSAppendOptionsPrefix(DS ds,const char *prefix)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  ierr = PetscObjectAppendOptionsPrefix((PetscObject)ds,prefix);CHKERRQ(ierr);
+  PetscCall(PetscObjectAppendOptionsPrefix((PetscObject)ds,prefix));
   PetscFunctionReturn(0);
 }
 
@@ -229,12 +216,10 @@ PetscErrorCode DSAppendOptionsPrefix(DS ds,const char *prefix)
 @*/
 PetscErrorCode DSGetOptionsPrefix(DS ds,const char *prefix[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidPointer(prefix,2);
-  ierr = PetscObjectGetOptionsPrefix((PetscObject)ds,prefix);CHKERRQ(ierr);
+  PetscCall(PetscObjectGetOptionsPrefix((PetscObject)ds,prefix));
   PetscFunctionReturn(0);
 }
 
@@ -243,7 +228,7 @@ PetscErrorCode DSGetOptionsPrefix(DS ds,const char *prefix[])
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds   - the direct solver context
 -  type - a known type
 
@@ -253,23 +238,23 @@ PetscErrorCode DSGetOptionsPrefix(DS ds,const char *prefix[])
 @*/
 PetscErrorCode DSSetType(DS ds,DSType type)
 {
-  PetscErrorCode ierr,(*r)(DS);
+  PetscErrorCode (*r)(DS);
   PetscBool      match;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidCharPointer(type,2);
 
-  ierr = PetscObjectTypeCompare((PetscObject)ds,type,&match);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)ds,type,&match));
   if (match) PetscFunctionReturn(0);
 
-  ierr =  PetscFunctionListFind(DSList,type,&r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested DS type %s",type);
+  PetscCall(PetscFunctionListFind(DSList,type,&r));
+  PetscCheck(r,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested DS type %s",type);
 
-  ierr = PetscMemzero(ds->ops,sizeof(struct _DSOps));CHKERRQ(ierr);
+  PetscCall(PetscMemzero(ds->ops,sizeof(struct _DSOps)));
 
-  ierr = PetscObjectChangeTypeName((PetscObject)ds,type);CHKERRQ(ierr);
-  ierr = (*r)(ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectChangeTypeName((PetscObject)ds,type));
+  PetscCall((*r)(ds));
   PetscFunctionReturn(0);
 }
 
@@ -282,7 +267,7 @@ PetscErrorCode DSSetType(DS ds,DSType type)
 .  ds - the direct solver context
 
    Output Parameter:
-.  name - name of the direct solver
+.  type - name of the direct solver
 
    Level: intermediate
 
@@ -321,15 +306,11 @@ PetscErrorCode DSGetType(DS ds,DSType *type)
 @*/
 PetscErrorCode DSDuplicate(DS ds,DS *dsnew)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidPointer(dsnew,2);
-  ierr = DSCreate(PetscObjectComm((PetscObject)ds),dsnew);CHKERRQ(ierr);
-  if (((PetscObject)ds)->type_name) {
-    ierr = DSSetType(*dsnew,((PetscObject)ds)->type_name);CHKERRQ(ierr);
-  }
+  PetscCall(DSCreate(PetscObjectComm((PetscObject)ds),dsnew));
+  if (((PetscObject)ds)->type_name) PetscCall(DSSetType(*dsnew,((PetscObject)ds)->type_name));
   (*dsnew)->method   = ds->method;
   (*dsnew)->compact  = ds->compact;
   (*dsnew)->refined  = ds->refined;
@@ -344,9 +325,9 @@ PetscErrorCode DSDuplicate(DS ds,DS *dsnew)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds   - the direct solver context
--  meth - an index indentifying the method
+-  meth - an index identifying the method
 
    Options Database Key:
 .  -ds_method <meth> - Sets the method
@@ -360,8 +341,8 @@ PetscErrorCode DSSetMethod(DS ds,PetscInt meth)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidLogicalCollectiveInt(ds,meth,2);
-  if (meth<0) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"The method must be a non-negative integer");
-  if (meth>DS_MAX_SOLVE) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Too large value for the method");
+  PetscCheck(meth>=0,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"The method must be a non-negative integer");
+  PetscCheck(meth<=DS_MAX_SOLVE,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Too large value for the method");
   ds->method = meth;
   PetscFunctionReturn(0);
 }
@@ -395,12 +376,13 @@ PetscErrorCode DSGetMethod(DS ds,PetscInt *meth)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds    - the direct solver context
 -  pmode - the parallel mode
 
    Options Database Key:
-.  -ds_parallel <mode> - Sets the parallel mode, either 'redundant' or 'synchronized'
+.  -ds_parallel <mode> - Sets the parallel mode, 'redundant', 'synchronized'
+   or 'distributed'
 
    Notes:
    In the 'redundant' parallel mode, all processes will make the computation
@@ -413,6 +395,10 @@ PetscErrorCode DSGetMethod(DS ds,PetscInt *meth)
    processes in the communicator. This communication is not done automatically,
    an explicit call to DSSynchronize() is required.
 
+   The 'distributed' parallel mode can be used in some DS types only, such
+   as the contour integral method of DSNEP. In this case, every MPI process
+   will be in charge of part of the computation.
+
    Level: advanced
 
 .seealso: DSSynchronize(), DSGetParallel()
@@ -455,7 +441,7 @@ PetscErrorCode DSGetParallel(DS ds,DSParallelType *pmode)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds   - the direct solver context
 -  comp - a boolean flag
 
@@ -510,7 +496,7 @@ PetscErrorCode DSGetCompact(DS ds,PetscBool *comp)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds  - the direct solver context
 -  ext - a boolean flag
 
@@ -531,7 +517,7 @@ PetscErrorCode DSSetExtraRow(DS ds,PetscBool ext)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidLogicalCollectiveBool(ds,ext,2);
-  if (ds->n>0 && ds->n==ds->ld) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot set extra row after setting n=ld");
+  PetscCheck(!ext || ds->n==0 || ds->n!=ds->ld,PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot set extra row after setting n=ld");
   ds->extrarow = ext;
   PetscFunctionReturn(0);
 }
@@ -566,7 +552,7 @@ PetscErrorCode DSGetExtraRow(DS ds,PetscBool *ext)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds  - the direct solver context
 -  ref - a boolean flag
 
@@ -622,7 +608,7 @@ PetscErrorCode DSGetRefined(DS ds,PetscBool *ref)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds - the direct solver context
 -  bs - the block size
 
@@ -638,7 +624,7 @@ PetscErrorCode DSSetBlockSize(DS ds,PetscInt bs)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidLogicalCollectiveInt(ds,bs,2);
-  if (bs<1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"The block size must be at least one");
+  PetscCheck(bs>0,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"The block size must be at least one");
   ds->bs = bs;
   PetscFunctionReturn(0);
 }
@@ -682,14 +668,10 @@ PetscErrorCode DSGetBlockSize(DS ds,PetscInt *bs)
 @*/
 PetscErrorCode DSSetSlepcSC(DS ds,SlepcSC sc)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidPointer(sc,2);
-  if (ds->sc && !ds->scset) {
-    ierr = PetscFree(ds->sc);CHKERRQ(ierr);
-  }
+  if (ds->sc && !ds->scset) PetscCall(PetscFree(ds->sc));
   ds->sc    = sc;
   ds->scset = PETSC_TRUE;
   PetscFunctionReturn(0);
@@ -712,14 +694,10 @@ PetscErrorCode DSSetSlepcSC(DS ds,SlepcSC sc)
 @*/
 PetscErrorCode DSGetSlepcSC(DS ds,SlepcSC *sc)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidPointer(sc,2);
-  if (!ds->sc) {
-    ierr = PetscNewLog(ds,&ds->sc);CHKERRQ(ierr);
-  }
+  if (!ds->sc) PetscCall(PetscNew(&ds->sc));
   *sc = ds->sc;
   PetscFunctionReturn(0);
 }
@@ -736,34 +714,34 @@ PetscErrorCode DSGetSlepcSC(DS ds,SlepcSC *sc)
    To see all options, run your program with the -help option.
 
    Level: beginner
+
+.seealso: DSSetOptionsPrefix()
 @*/
 PetscErrorCode DSSetFromOptions(DS ds)
 {
-  PetscErrorCode ierr;
   PetscInt       bs,meth;
   PetscBool      flag;
   DSParallelType pmode;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  ierr = DSRegisterAll();CHKERRQ(ierr);
+  PetscCall(DSRegisterAll());
   /* Set default type (we do not allow changing it with -ds_type) */
-  if (!((PetscObject)ds)->type_name) {
-    ierr = DSSetType(ds,DSNHEP);CHKERRQ(ierr);
-  }
-  ierr = PetscObjectOptionsBegin((PetscObject)ds);CHKERRQ(ierr);
+  if (!((PetscObject)ds)->type_name) PetscCall(DSSetType(ds,DSNHEP));
+  PetscObjectOptionsBegin((PetscObject)ds);
 
-    ierr = PetscOptionsInt("-ds_block_size","Block size for the dense system solver","DSSetBlockSize",ds->bs,&bs,&flag);CHKERRQ(ierr);
-    if (flag) { ierr = DSSetBlockSize(ds,bs);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsInt("-ds_block_size","Block size for the dense system solver","DSSetBlockSize",ds->bs,&bs,&flag));
+    if (flag) PetscCall(DSSetBlockSize(ds,bs));
 
-    ierr = PetscOptionsInt("-ds_method","Method to be used for the dense system","DSSetMethod",ds->method,&meth,&flag);CHKERRQ(ierr);
-    if (flag) { ierr = DSSetMethod(ds,meth);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsInt("-ds_method","Method to be used for the dense system","DSSetMethod",ds->method,&meth,&flag));
+    if (flag) PetscCall(DSSetMethod(ds,meth));
 
-    ierr = PetscOptionsEnum("-ds_parallel","Operation mode in parallel runs","DSSetParallel",DSParallelTypes,(PetscEnum)ds->pmode,(PetscEnum*)&pmode,&flag);CHKERRQ(ierr);
-    if (flag) { ierr = DSSetParallel(ds,pmode);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsEnum("-ds_parallel","Operation mode in parallel runs","DSSetParallel",DSParallelTypes,(PetscEnum)ds->pmode,(PetscEnum*)&pmode,&flag));
+    if (flag) PetscCall(DSSetParallel(ds,pmode));
 
-    ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)ds);CHKERRQ(ierr);
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+    PetscTryTypeMethod(ds,setfromoptions,PetscOptionsObject);
+    PetscCall(PetscObjectProcessOptionsHandlers((PetscObject)ds,PetscOptionsObject));
+  PetscOptionsEnd();
   PetscFunctionReturn(0);
 }
 
@@ -793,46 +771,31 @@ PetscErrorCode DSSetFromOptions(DS ds)
 @*/
 PetscErrorCode DSView(DS ds,PetscViewer viewer)
 {
-  PetscBool         isascii,issvd;
+  PetscBool         isascii;
   PetscViewerFormat format;
-  PetscErrorCode    ierr;
   PetscMPIInt       size;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ds),&viewer);CHKERRQ(ierr);
-  }
+  if (!viewer) PetscCall(PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ds),&viewer));
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(ds,1,viewer,2);
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii));
   if (isascii) {
-    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
-    ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ds,viewer);CHKERRQ(ierr);
-    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRMPI(ierr);
-    if (size>1) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",DSParallelTypes[ds->pmode]);CHKERRQ(ierr);
-    }
+    PetscCall(PetscViewerGetFormat(viewer,&format));
+    PetscCall(PetscObjectPrintClassNamePrefixType((PetscObject)ds,viewer));
+    PetscCallMPI(MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size));
+    if (size>1) PetscCall(PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",DSParallelTypes[ds->pmode]));
     if (format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  current state: %s\n",DSStateTypes[ds->state]);CHKERRQ(ierr);
-      ierr = PetscObjectTypeCompare((PetscObject)ds,DSSVD,&issvd);CHKERRQ(ierr);
-      if (issvd) {
-        ierr = PetscViewerASCIIPrintf(viewer,"  dimensions: ld=%D, n=%D, m=%D, l=%D, k=%D",ds->ld,ds->n,ds->m,ds->l,ds->k);CHKERRQ(ierr);
-      } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"  dimensions: ld=%D, n=%D, l=%D, k=%D",ds->ld,ds->n,ds->l,ds->k);CHKERRQ(ierr);
-      }
-      if (ds->state==DS_STATE_TRUNCATED) {
-        ierr = PetscViewerASCIIPrintf(viewer,", t=%D\n",ds->t);CHKERRQ(ierr);
-      } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
-      }
-      ierr = PetscViewerASCIIPrintf(viewer,"  flags:%s%s%s\n",ds->compact?" compact":"",ds->extrarow?" extrarow":"",ds->refined?" refined":"");CHKERRQ(ierr);
-    }
-    if (ds->ops->view) {
-      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-      ierr = (*ds->ops->view)(ds,viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"  current state: %s\n",DSStateTypes[ds->state]));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"  dimensions: ld=%" PetscInt_FMT ", n=%" PetscInt_FMT ", l=%" PetscInt_FMT ", k=%" PetscInt_FMT,ds->ld,ds->n,ds->l,ds->k));
+      if (ds->state==DS_STATE_TRUNCATED) PetscCall(PetscViewerASCIIPrintf(viewer,", t=%" PetscInt_FMT "\n",ds->t));
+      else PetscCall(PetscViewerASCIIPrintf(viewer,"\n"));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"  flags:%s%s%s\n",ds->compact?" compact":"",ds->extrarow?" extrarow":"",ds->refined?" refined":""));
     }
+    PetscCall(PetscViewerASCIIPushTab(viewer));
+    PetscTryTypeMethod(ds,view,viewer);
+    PetscCall(PetscViewerASCIIPopTab(viewer));
   }
   PetscFunctionReturn(0);
 }
@@ -853,11 +816,9 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
 @*/
 PetscErrorCode DSViewFromOptions(DS ds,PetscObject obj,const char name[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  ierr = PetscObjectViewFromOptions((PetscObject)ds,obj,name);CHKERRQ(ierr);
+  PetscCall(PetscObjectViewFromOptions((PetscObject)ds,obj,name));
   PetscFunctionReturn(0);
 }
 
@@ -881,18 +842,16 @@ PetscErrorCode DSViewFromOptions(DS ds,PetscObject obj,const char name[])
 @*/
 PetscErrorCode DSAllocate(DS ds,PetscInt ld)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidLogicalCollectiveInt(ds,ld,2);
   PetscValidType(ds,1);
-  if (ld<1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Leading dimension should be at least one");
+  PetscCheck(ld>0,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Leading dimension should be at least one");
   if (ld!=ds->ld) {
-    ierr = PetscInfo1(ds,"Allocating memory with leading dimension=%D\n",ld);CHKERRQ(ierr);
-    ierr = DSReset(ds);CHKERRQ(ierr);
+    PetscCall(PetscInfo(ds,"Allocating memory with leading dimension=%" PetscInt_FMT "\n",ld));
+    PetscCall(DSReset(ds));
     ds->ld = ld;
-    ierr = (*ds->ops->allocate)(ds,ld);CHKERRQ(ierr);
+    PetscUseTypeMethod(ds,allocate,ld);
   }
   PetscFunctionReturn(0);
 }
@@ -916,7 +875,6 @@ PetscErrorCode DSAllocate(DS ds,PetscInt ld)
 PetscErrorCode DSReset(DS ds)
 {
   PetscInt       i;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (ds) PetscValidHeaderSpecific(ds,DS_CLASSID,1);
@@ -925,14 +883,9 @@ PetscErrorCode DSReset(DS ds)
   ds->ld       = 0;
   ds->l        = 0;
   ds->n        = 0;
-  ds->m        = 0;
   ds->k        = 0;
-  for (i=0;i<DS_NUM_MAT;i++) {
-    ierr = PetscFree(ds->mat[i]);CHKERRQ(ierr);
-    ierr = PetscFree(ds->rmat[i]);CHKERRQ(ierr);
-    ierr = MatDestroy(&ds->omat[i]);CHKERRQ(ierr);
-  }
-  ierr = PetscFree(ds->perm);CHKERRQ(ierr);
+  for (i=0;i<DS_NUM_MAT;i++) PetscCall(MatDestroy(&ds->omat[i]));
+  PetscCall(PetscFree(ds->perm));
   PetscFunctionReturn(0);
 }
 
@@ -950,19 +903,17 @@ PetscErrorCode DSReset(DS ds)
 @*/
 PetscErrorCode DSDestroy(DS *ds)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   if (!*ds) PetscFunctionReturn(0);
   PetscValidHeaderSpecific(*ds,DS_CLASSID,1);
-  if (--((PetscObject)(*ds))->refct > 0) { *ds = 0; PetscFunctionReturn(0); }
-  ierr = DSReset(*ds);CHKERRQ(ierr);
-  if ((*ds)->ops->destroy) { ierr = (*(*ds)->ops->destroy)(*ds);CHKERRQ(ierr); }
-  ierr = PetscFree((*ds)->work);CHKERRQ(ierr);
-  ierr = PetscFree((*ds)->rwork);CHKERRQ(ierr);
-  ierr = PetscFree((*ds)->iwork);CHKERRQ(ierr);
-  if (!(*ds)->scset) { ierr = PetscFree((*ds)->sc);CHKERRQ(ierr); }
-  ierr = PetscHeaderDestroy(ds);CHKERRQ(ierr);
+  if (--((PetscObject)(*ds))->refct > 0) { *ds = NULL; PetscFunctionReturn(0); }
+  PetscCall(DSReset(*ds));
+  PetscTryTypeMethod(*ds,destroy);
+  PetscCall(PetscFree((*ds)->work));
+  PetscCall(PetscFree((*ds)->rwork));
+  PetscCall(PetscFree((*ds)->iwork));
+  if (!(*ds)->scset) PetscCall(PetscFree((*ds)->sc));
+  PetscCall(PetscHeaderDestroy(ds));
   PetscFunctionReturn(0);
 }
 
@@ -973,7 +924,7 @@ PetscErrorCode DSDestroy(DS *ds)
 
    Input Parameters:
 +  name - name of a new user-defined DS
--  routine_create - routine to create context
+-  function - routine to create context
 
    Notes:
    DSRegister() may be called multiple times to add several user-defined
@@ -985,11 +936,9 @@ PetscErrorCode DSDestroy(DS *ds)
 @*/
 PetscErrorCode DSRegister(const char *name,PetscErrorCode (*function)(DS))
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
-  ierr = DSInitializePackage();CHKERRQ(ierr);
-  ierr = PetscFunctionListAdd(&DSList,name,function);CHKERRQ(ierr);
+  PetscCall(DSInitializePackage());
+  PetscCall(PetscFunctionListAdd(&DSList,name,function));
   PetscFunctionReturn(0);
 }
 
@@ -1000,6 +949,7 @@ SLEPC_EXTERN PetscErrorCode DSCreate_GHIEP(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_GNHEP(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_NHEPTS(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_SVD(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_GSVD(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_PEP(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_NEP(DS);
 
@@ -1009,23 +959,23 @@ SLEPC_EXTERN PetscErrorCode DSCreate_NEP(DS);
    Not Collective
 
    Level: advanced
+
+.seealso: DSRegister()
 @*/
 PetscErrorCode DSRegisterAll(void)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   if (DSRegisterAllCalled) PetscFunctionReturn(0);
   DSRegisterAllCalled = PETSC_TRUE;
-  ierr = DSRegister(DSHEP,DSCreate_HEP);CHKERRQ(ierr);
-  ierr = DSRegister(DSNHEP,DSCreate_NHEP);CHKERRQ(ierr);
-  ierr = DSRegister(DSGHEP,DSCreate_GHEP);CHKERRQ(ierr);
-  ierr = DSRegister(DSGHIEP,DSCreate_GHIEP);CHKERRQ(ierr);
-  ierr = DSRegister(DSGNHEP,DSCreate_GNHEP);CHKERRQ(ierr);
-  ierr = DSRegister(DSNHEPTS,DSCreate_NHEPTS);CHKERRQ(ierr);
-  ierr = DSRegister(DSSVD,DSCreate_SVD);CHKERRQ(ierr);
-  ierr = DSRegister(DSPEP,DSCreate_PEP);CHKERRQ(ierr);
-  ierr = DSRegister(DSNEP,DSCreate_NEP);CHKERRQ(ierr);
+  PetscCall(DSRegister(DSHEP,DSCreate_HEP));
+  PetscCall(DSRegister(DSNHEP,DSCreate_NHEP));
+  PetscCall(DSRegister(DSGHEP,DSCreate_GHEP));
+  PetscCall(DSRegister(DSGHIEP,DSCreate_GHIEP));
+  PetscCall(DSRegister(DSGNHEP,DSCreate_GNHEP));
+  PetscCall(DSRegister(DSNHEPTS,DSCreate_NHEPTS));
+  PetscCall(DSRegister(DSSVD,DSCreate_SVD));
+  PetscCall(DSRegister(DSGSVD,DSCreate_GSVD));
+  PetscCall(DSRegister(DSPEP,DSCreate_PEP));
+  PetscCall(DSRegister(DSNEP,DSCreate_NEP));
   PetscFunctionReturn(0);
 }
-
