@@ -224,7 +224,7 @@ PetscErrorCode SlepcCitationsInitialize()
   PetscCall(PetscCitationsRegister("@TechReport{slepc-manual,\n"
     "   author = \"J. E. Roman and C. Campos and L. Dalcin and E. Romero and A. Tomas\",\n"
     "   title = \"{SLEPc} Users Manual\",\n"
-    "   number = \"DSIC-II/24/02 - Revision 3.17\",\n"
+    "   number = \"DSIC-II/24/02 - Revision 3.18\",\n"
     "   institution = \"D. Sistemes Inform\\`atics i Computaci\\'o, Universitat Polit\\`ecnica de Val\\`encia\",\n"
     "   year = \"2022\"\n"
     "}\n",NULL));
@@ -319,7 +319,7 @@ PetscErrorCode SlepcFinalize(void)
 PetscErrorCode SlepcInitializeNoArguments(void)
 {
   int  argc = 0;
-  char **args = 0;
+  char **args = NULL;
 
   PetscFunctionBegin;
   PetscCall(SlepcInitialize(&argc,&args,NULL,NULL));
